"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BalancesCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
class BalancesCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(BalancesCommand);
        const client = await this.sdk.connectRpc();
        const account = args.account;
        const response = await client.wallet.getAccountBalances({
            account,
            confirmations: flags.confirmations,
        });
        this.log(`Account: ${response.content.account}`);
        const assetBalancePairs = [];
        for (const balance of response.content.balances) {
            const asset = await client.wallet.getAsset({
                account,
                id: balance.assetId,
                confirmations: flags.confirmations,
            });
            assetBalancePairs.push({
                balance,
                asset: asset.content,
            });
        }
        let columns = {
            assetName: {
                header: 'Asset Name',
                get: ({ asset }) => (0, utils_1.renderAssetWithVerificationStatus)(sdk_1.BufferUtils.toHuman(Buffer.from(asset.name, 'hex')), {
                    verification: asset.verification,
                    outputType: flags.output,
                }),
            },
            'asset.id': {
                header: 'Asset Id',
                get: ({ asset }) => asset.id,
            },
            available: {
                header: 'Available Balance',
                get: ({ asset, balance }) => sdk_1.CurrencyUtils.render(balance.available, false, asset.id, asset.verification),
            },
        };
        if (flags.all) {
            columns = {
                ...columns,
                confirmed: {
                    header: 'Confirmed Balance',
                    get: ({ asset, balance }) => sdk_1.CurrencyUtils.render(balance.confirmed, false, asset.id, asset.verification),
                },
                unconfirmed: {
                    header: 'Unconfirmed Balance',
                    get: ({ asset, balance }) => sdk_1.CurrencyUtils.render(balance.unconfirmed, false, asset.id, asset.verification),
                },
                pending: {
                    header: 'Pending Balance',
                    get: ({ asset, balance }) => sdk_1.CurrencyUtils.render(balance.pending, false, asset.id, asset.verification),
                },
                blockHash: {
                    header: 'Head Hash',
                    get: ({ balance }) => balance.blockHash || 'NULL',
                },
                sequence: {
                    header: 'Head Sequence',
                    get: ({ balance }) => balance.sequence || 'NULL',
                },
            };
        }
        assetBalancePairs.sort((left, right) => (0, utils_1.compareAssets)(left.asset.name, left.asset.verification, right.asset.name, right.asset.verification));
        core_1.CliUx.ux.table(assetBalancePairs, columns, flags);
    }
}
BalancesCommand.description = `Display the account's balances for all assets`;
BalancesCommand.flags = {
    ...flags_1.RemoteFlags,
    ...core_1.CliUx.ux.table.flags(),
    all: core_1.Flags.boolean({
        default: false,
        description: `Also show unconfirmed balance, head hash, and head sequence`,
    }),
    confirmations: core_1.Flags.integer({
        required: false,
        description: 'Minimum number of blocks confirmations for a transaction',
    }),
};
BalancesCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'Name of the account to get balances for',
    },
];
exports.BalancesCommand = BalancesCommand;
//# sourceMappingURL=balances.js.map