"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateSigningCommitmentCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../../../command");
const flags_1 = require("../../../../flags");
const longPrompt_1 = require("../../../../utils/longPrompt");
const multisig_1 = require("../../../../utils/multisig");
const transaction_1 = require("../../../../utils/transaction");
class CreateSigningCommitmentCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(CreateSigningCommitmentCommand);
        const loaded = await multisig_1.MultisigTransactionJson.load(this.sdk.fileSystem, flags.path);
        const options = multisig_1.MultisigTransactionJson.resolveFlags(flags, loaded);
        let identities = options.identity;
        if (!identities || identities.length < 2) {
            const input = await (0, longPrompt_1.longPrompt)('Enter the identities of all participants who will sign the transaction, separated by commas', {
                required: true,
            });
            identities = input.split(',');
            if (identities.length < 2) {
                this.error('Minimum number of identities must be at least 2');
            }
        }
        identities = identities.map((i) => i.trim());
        let unsignedTransactionInput = options.unsignedTransaction;
        if (!unsignedTransactionInput) {
            unsignedTransactionInput = await (0, longPrompt_1.longPrompt)('Enter the unsigned transaction', {
                required: true,
            });
        }
        const client = await this.sdk.connectRpc();
        const unsignedTransaction = new sdk_1.UnsignedTransaction(Buffer.from(unsignedTransactionInput, 'hex'));
        await (0, transaction_1.renderUnsignedTransactionDetails)(client, unsignedTransaction, flags.account, this.logger);
        if (!flags.confirm) {
            const confirmed = await core_1.CliUx.ux.confirm('Confirm signing commitment creation (Y/N)');
            if (!confirmed) {
                this.error('Creating signing commitment aborted');
            }
        }
        const response = await client.wallet.multisig.createSigningCommitment({
            account: flags.account,
            unsignedTransaction: unsignedTransactionInput,
            signers: identities.map((identity) => ({ identity })),
        });
        this.log('\nCommitment:\n');
        this.log(response.content.commitment);
        this.log();
        this.log('Next step:');
        this.log('Send the commitment to the multisig account coordinator.');
    }
}
CreateSigningCommitmentCommand.description = 'Create a signing commitment from a participant for a given transaction';
CreateSigningCommitmentCommand.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to use for generating the commitment, must be a multisig participant account',
        required: false,
    }),
    unsignedTransaction: core_1.Flags.string({
        char: 'u',
        description: 'The unsigned transaction that needs to be signed',
    }),
    identity: core_1.Flags.string({
        char: 'i',
        description: 'The identity of the participants that will sign the transaction (may be specified multiple times to add multiple signers)',
        multiple: true,
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm creating signing commitment without confirming',
    }),
    path: core_1.Flags.string({
        description: 'Path to a JSON file containing multisig transaction data',
    }),
};
exports.CreateSigningCommitmentCommand = CreateSigningCommitmentCommand;
//# sourceMappingURL=create.js.map