"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultisigCreateDealer = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../../../command");
const flags_1 = require("../../../../flags");
const longPrompt_1 = require("../../../../utils/longPrompt");
class MultisigCreateDealer extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(MultisigCreateDealer);
        let identities = flags.identity;
        if (!identities || identities.length < 2) {
            const input = await (0, longPrompt_1.longPrompt)('Enter the identities of all participants, separated by commas', {
                required: true,
            });
            identities = input.split(',');
            if (identities.length < 2) {
                this.error('Minimum number of identities must be at least 2');
            }
        }
        identities = identities.map((i) => i.trim());
        let minSigners = flags.minSigners;
        if (!minSigners) {
            const input = await core_1.CliUx.ux.prompt('Enter the number of minimum signers', {
                required: true,
            });
            minSigners = parseInt(input);
            if (isNaN(minSigners) || minSigners < 2) {
                this.error('Minimum number of signers must be at least 2');
            }
        }
        const client = await this.sdk.connectRpc();
        const name = await this.getCoordinatorName(client, flags.name?.trim());
        const response = await client.wallet.multisig.createTrustedDealerKeyPackage({
            minSigners,
            participants: identities.map((identity) => ({ identity })),
        });
        const chainResponse = await client.chain.getChainInfo();
        const hash = Buffer.from(chainResponse.content.currentBlockIdentifier.hash, 'hex');
        const sequence = Number(chainResponse.content.currentBlockIdentifier.index);
        const createdAt = {
            hash,
            sequence,
        };
        if (flags.importCoordinator) {
            this.log();
            core_1.CliUx.ux.action.start('Importing the coordinator as a view-only account');
            await client.wallet.importAccount({
                account: {
                    name,
                    version: sdk_1.ACCOUNT_SCHEMA_VERSION,
                    createdAt: {
                        hash: createdAt.hash.toString('hex'),
                        sequence: createdAt.sequence,
                    },
                    spendingKey: null,
                    viewKey: response.content.viewKey,
                    incomingViewKey: response.content.incomingViewKey,
                    outgoingViewKey: response.content.outgoingViewKey,
                    publicAddress: response.content.publicAddress,
                    proofAuthorizingKey: response.content.proofAuthorizingKey,
                    multisigKeys: {
                        publicKeyPackage: response.content.publicKeyPackage,
                    },
                },
            });
            core_1.CliUx.ux.action.stop();
        }
        for (const [i, { identity, account }] of response.content.participantAccounts.entries()) {
            this.log('\n');
            this.log(`Account ${i + 1}`);
            this.log(`Identity ${identity}`);
            this.log('----------------');
            this.log(account);
        }
        this.log();
        this.log('Next step:');
        this.log('Send the account imports to the participant with the corresponding identity.');
    }
    async getCoordinatorName(client, inputName) {
        const accountsResponse = await client.wallet.getAccounts();
        const accountNames = new Set(accountsResponse.content.accounts);
        const identitiesResponse = await client.wallet.multisig.getIdentities();
        const secretNames = new Set(identitiesResponse.content.identities.map((i) => i.name));
        let name = inputName;
        do {
            name =
                name ?? (await core_1.CliUx.ux.prompt('Enter a name for the coordinator', { required: true }));
            if (accountNames.has(name)) {
                this.log(`Account with name ${name} already exists`);
                this.log('');
                name = undefined;
            }
            else if (secretNames.has(name)) {
                this.log(`Multisig identity with name ${name} already exists`);
                this.log('');
                name = undefined;
            }
        } while (name === undefined);
        return name;
    }
}
MultisigCreateDealer.description = `Create a set of multisig accounts from participant identities`;
MultisigCreateDealer.flags = {
    ...flags_1.RemoteFlags,
    name: core_1.Flags.string({
        char: 'n',
        description: 'Name to use for the coordinator',
    }),
    identity: core_1.Flags.string({
        char: 'i',
        description: 'Identity of a participant',
        multiple: true,
    }),
    minSigners: core_1.Flags.integer({
        char: 'm',
        description: 'Minimum number of signers to meet signing threshold',
    }),
    importCoordinator: core_1.Flags.boolean({
        char: 'c',
        default: true,
        description: 'Import the coordinator as a view-only account after creating key packages',
    }),
};
exports.MultisigCreateDealer = MultisigCreateDealer;
//# sourceMappingURL=create.js.map