"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateSignatureShareCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../../../command");
const flags_1 = require("../../../../flags");
const longPrompt_1 = require("../../../../utils/longPrompt");
const multisig_1 = require("../../../../utils/multisig");
const transaction_1 = require("../../../../utils/transaction");
class CreateSignatureShareCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(CreateSignatureShareCommand);
        const loaded = await multisig_1.MultisigTransactionJson.load(this.sdk.fileSystem, flags.path);
        const options = multisig_1.MultisigTransactionJson.resolveFlags(flags, loaded);
        let signingPackageString = options.signingPackage;
        if (!signingPackageString) {
            signingPackageString = await (0, longPrompt_1.longPrompt)('Enter the signing package');
        }
        const client = await this.sdk.connectRpc();
        const signingPackage = new rust_nodejs_1.multisig.SigningPackage(Buffer.from(signingPackageString, 'hex'));
        const unsignedTransaction = new sdk_1.UnsignedTransaction(signingPackage.unsignedTransaction().serialize());
        this.renderSigners(signingPackage.signers());
        await (0, transaction_1.renderUnsignedTransactionDetails)(client, unsignedTransaction, flags.account, this.logger);
        if (!flags.confirm) {
            const confirmed = await core_1.CliUx.ux.confirm('Confirm new signature share creation (Y/N)');
            if (!confirmed) {
                this.error('Creating signature share aborted');
            }
        }
        const signatureShareResponse = await client.wallet.multisig.createSignatureShare({
            account: flags.account,
            signingPackage: signingPackageString,
        });
        this.log();
        this.log('Signature Share:');
        this.log(signatureShareResponse.content.signatureShare);
        this.log();
        this.log('Next step:');
        this.log('Send the signature to the coordinator. They will aggregate the signatures from all participants and sign the transaction.');
    }
    renderSigners(signers) {
        this.log('');
        this.log('==================');
        this.log('Signer Identities:');
        this.log('==================');
        for (const [i, signer] of signers.entries()) {
            if (i !== 0) {
                this.log('------------------');
            }
            this.log(signer.toString('hex'));
        }
    }
}
CreateSignatureShareCommand.description = `Creates a signature share for a participant for a given transaction`;
CreateSignatureShareCommand.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account from which the signature share will be created',
        required: false,
    }),
    signingPackage: core_1.Flags.string({
        char: 's',
        description: 'The signing package for which the signature share will be created',
        required: false,
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm creating signature share without confirming',
    }),
    path: core_1.Flags.string({
        description: 'Path to a JSON file containing multisig transaction data',
    }),
};
exports.CreateSignatureShareCommand = CreateSignatureShareCommand;
//# sourceMappingURL=create.js.map