"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CombineNotesCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const inquirer_1 = __importDefault(require("inquirer"));
const command_1 = require("../../../command");
const flags_1 = require("../../../flags");
const utils_1 = require("../../../utils");
const explorer_1 = require("../../../utils/explorer");
const fees_1 = require("../../../utils/fees");
const note_1 = require("../../../utils/note");
const spendPostTime_1 = require("../../../utils/spendPostTime");
const transaction_1 = require("../../../utils/transaction");
class CombineNotesCommand extends command_1.IronfishCommand {
    async selectNotesToCombine(spendPostTimeMs) {
        const spendsPerMinute = Math.max(Math.floor(60000 / spendPostTimeMs), 2); // minimum of 2 notes per minute in case the spentPostTime is very high
        const low = spendsPerMinute;
        const medium = spendsPerMinute * 5;
        const high = spendsPerMinute * 10;
        const choices = [
            {
                name: `~1 minute: ${low} notes`,
                value: low,
            },
            {
                name: `~5 minutes: ${medium} notes`,
                value: medium,
                default: true,
            },
            {
                name: `~10 minutes: ${high} notes`,
                value: high,
            },
            {
                name: 'Enter a custom number of notes',
                value: null,
            },
        ];
        const result = await inquirer_1.default.prompt([
            {
                name: 'selection',
                message: `Select the number of notes you wish to combine (MAX): `,
                type: 'list',
                choices,
            },
        ]);
        if (result.selection) {
            return result.selection;
        }
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const result = await core_1.CliUx.ux.prompt('Enter the number of notes', {
                required: true,
            });
            const notesToCombine = parseInt(result);
            if (isNaN(notesToCombine)) {
                this.logger.error(`The number of notes must be a number`);
                continue;
            }
            if (notesToCombine > high) {
                this.logger.error(`The number of notes cannot be higher than the ${high}`);
                continue;
            }
            if (notesToCombine < 2) {
                this.logger.error(`The number must be larger than 1`);
                continue;
            }
            return notesToCombine;
        }
    }
    async calculateExpiration(client, spendPostTimeInMs, numberOfNotes) {
        const currentBlockSequence = await this.getCurrentBlockSequence(client);
        let timeLastFiveBlocksInMs = 0;
        let currentBlockTime = new Date((await client.chain.getBlock({
            sequence: currentBlockSequence,
        })).content.block.timestamp);
        for (let i = 0; i < 5; i++) {
            const blockTime = new Date((await client.chain.getBlock({
                sequence: currentBlockSequence - i,
            })).content.block.timestamp);
            timeLastFiveBlocksInMs += currentBlockTime.getTime() - blockTime.getTime();
            currentBlockTime = blockTime;
        }
        const averageBlockTimeInMs = timeLastFiveBlocksInMs / 5;
        const targetBlockTimeInMs = (await client.chain.getConsensusParameters()).content.targetBlockTimeInSeconds * 1000;
        const blockTimeForCalculation = Math.min(averageBlockTimeInMs, targetBlockTimeInMs);
        let expiration = Math.ceil(currentBlockSequence + (spendPostTimeInMs * numberOfNotes * 2) / blockTimeForCalculation);
        const config = await client.config.getConfig();
        if (config.content.transactionExpirationDelta) {
            expiration = Math.max(currentBlockSequence + config.content.transactionExpirationDelta, expiration);
        }
        return expiration;
    }
    async getCurrentBlockSequence(client) {
        const getCurrentBlock = await client.chain.getChainInfo();
        const currentBlockSequence = parseInt(getCurrentBlock.content.currentBlockIdentifier.index);
        return currentBlockSequence;
    }
    async start() {
        const { flags } = await this.parse(CombineNotesCommand);
        const client = await this.sdk.connectRpc();
        let to = flags.to;
        let from = flags.account;
        if (!from) {
            const response = await client.wallet.getDefaultAccount();
            if (!response.content.account) {
                this.error(`No account is currently active.
          Use ironfish wallet:create <name> to first create an account`);
            }
            from = response.content.account.name;
        }
        if (!to) {
            const response = await client.wallet.getAccountPublicKey({
                account: from,
            });
            to = response.content.publicKey;
        }
        await this.ensureUserHasEnoughNotesToCombine(client, from);
        let spendPostTime = (0, spendPostTime_1.getSpendPostTimeInMs)(this.sdk);
        if (spendPostTime <= 0 || flags.benchmark) {
            spendPostTime = await (0, spendPostTime_1.benchmarkSpendPostTime)(this.sdk, client, from);
        }
        let numberOfNotes = flags.notes;
        if (numberOfNotes === undefined) {
            numberOfNotes = await this.selectNotesToCombine(spendPostTime);
        }
        let notes = await (0, note_1.fetchNotes)(client, from, numberOfNotes);
        // If the user doesn't have enough notes for their selection, we reduce the number of notes so that
        // the largest note can be used for fees.
        if (notes.length < numberOfNotes) {
            numberOfNotes = notes.length - 1;
        }
        notes = notes.slice(0, numberOfNotes);
        const amountIncludingFees = notes.reduce((acc, note) => acc + BigInt(note.value), 0n);
        const memo = flags.memo?.trim() ??
            (await core_1.CliUx.ux.prompt('Enter the memo (or leave blank)', { required: false }));
        const expiration = await this.calculateExpiration(client, spendPostTime, numberOfNotes);
        const params = {
            account: from,
            outputs: [
                {
                    publicAddress: to,
                    amount: sdk_1.CurrencyUtils.encode(amountIncludingFees),
                    memo,
                },
            ],
            fee: flags.fee ? sdk_1.CurrencyUtils.encode(flags.fee) : null,
            feeRate: flags.feeRate ? sdk_1.CurrencyUtils.encode(flags.feeRate) : null,
            notes: notes.map((note) => note.noteHash),
            expiration,
        };
        let raw;
        if (params.fee === null && params.feeRate === null) {
            raw = await (0, fees_1.selectFee)({
                client,
                transaction: params,
                account: from,
                logger: this.logger,
            });
        }
        else {
            const response = await client.wallet.createTransaction(params);
            const bytes = Buffer.from(response.content.transaction, 'hex');
            raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        }
        // This allows for a single note output.
        const amount = amountIncludingFees - raw.fee;
        params.outputs[0].amount = sdk_1.CurrencyUtils.encode(amount);
        params.fee = sdk_1.CurrencyUtils.encode(raw.fee);
        const createTransactionResponse = await client.wallet.createTransaction(params);
        const createTransactionBytes = Buffer.from(createTransactionResponse.content.transaction, 'hex');
        raw = sdk_1.RawTransactionSerde.deserialize(createTransactionBytes);
        // TODO(mat): We need to add asset support here for bridges, etc.
        const assetData = (await client.wallet.getAsset({
            id: rust_nodejs_1.Asset.nativeId().toString('hex'),
            account: from,
        })).content;
        (0, transaction_1.displayTransactionSummary)(raw, assetData, amount, from, to, memo);
        const transactionTimer = new transaction_1.TransactionTimer(spendPostTime, raw);
        this.log(`Time to combine: ${sdk_1.TimeUtils.renderSpan(transactionTimer.getEstimateInMs(), {
            hideMilliseconds: true,
        })}`);
        await (0, utils_1.confirmOperation)({
            confirm: flags.confirm,
            cancelledMessage: 'Combine aborted.',
        });
        transactionTimer.start();
        const response = await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            account: from,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        transactionTimer.end();
        await (0, spendPostTime_1.updateSpendPostTimeInMs)(this.sdk, raw, transactionTimer.getStartTime(), transactionTimer.getEndTime());
        this.log(`Combining took ${sdk_1.TimeUtils.renderSpan(transactionTimer.getEndTime() - transactionTimer.getStartTime(), {
            hideMilliseconds: true,
        })}`);
        if (response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        await this.displayCombinedNoteHashes(client, from, transaction, assetData);
        this.log(`Transaction hash: ${transaction.hash().toString('hex')}`);
        const networkId = (await client.chain.getNetworkInfo()).content.networkId;
        const transactionUrl = (0, explorer_1.getExplorer)(networkId)?.getTransactionUrl(transaction.hash().toString('hex'));
        if (transactionUrl) {
            this.log(`\nIf the transaction is mined, it will appear here: ${transactionUrl}`);
        }
        if (flags.watch) {
            this.log('');
            await (0, transaction_1.watchTransaction)({
                client,
                logger: this.logger,
                account: from,
                hash: transaction.hash().toString('hex'),
            });
        }
    }
    async ensureUserHasEnoughNotesToCombine(client, from) {
        const notes = await (0, note_1.fetchNotes)(client, from, 10);
        if (notes.length < 3) {
            this.log(`Your notes are already combined. You currently have ${notes.length} notes.`);
            this.exit(0);
        }
    }
    async displayCombinedNoteHashes(client, from, transaction, assetData) {
        const resultingNotes = (await client.wallet.getAccountTransaction({
            account: from,
            hash: transaction.hash().toString('hex'),
        })).content.transaction?.notes;
        if (resultingNotes) {
            this.log('');
            core_1.CliUx.ux.table(resultingNotes, {
                hash: {
                    header: 'Notes Created',
                    get: (note) => note.noteHash,
                },
                value: {
                    header: 'Value',
                    get: (note) => sdk_1.CurrencyUtils.render(note.value, true, assetData.id, assetData.verification),
                },
                owner: {
                    header: 'Owner',
                    get: (note) => note.owner,
                },
            });
            this.log('');
        }
    }
}
CombineNotesCommand.description = `Combine notes into a single note`;
CombineNotesCommand.flags = {
    ...flags_1.RemoteFlags,
    fee: (0, flags_1.IronFlag)({
        char: 'o',
        description: 'The fee amount in IRON',
        minimum: 1n,
        flagName: 'fee',
    }),
    feeRate: (0, flags_1.IronFlag)({
        char: 'r',
        description: 'The fee rate amount in IRON/Kilobyte',
        minimum: 1n,
        flagName: 'fee rate',
    }),
    memo: core_1.Flags.string({
        char: 'm',
        description: 'The memo of transaction',
    }),
    notes: core_1.Flags.integer({
        description: 'How many notes to combine',
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    watch: core_1.Flags.boolean({
        default: false,
        description: 'Wait for the transaction to be confirmed',
    }),
    to: core_1.Flags.string({
        char: 't',
        description: 'The public address of the recipient',
    }),
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to send money from',
    }),
    benchmark: core_1.Flags.boolean({
        hidden: true,
        default: false,
        description: 'Force run the benchmark to measure the time to combine 1 note',
    }),
};
exports.CombineNotesCommand = CombineNotesCommand;
//# sourceMappingURL=combine.js.map