"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Send = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
const asset_1 = require("../../utils/asset");
const currency_1 = require("../../utils/currency");
const explorer_1 = require("../../utils/explorer");
const fees_1 = require("../../utils/fees");
const spendPostTime_1 = require("../../utils/spendPostTime");
const transaction_1 = require("../../utils/transaction");
class Send extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Send);
        let assetId = flags.assetId;
        let to = flags.to?.trim();
        let from = flags.account?.trim();
        const client = await this.sdk.connectRpc();
        if (!flags.offline) {
            const status = await client.wallet.getNodeStatus();
            if (!status.content.blockchain.synced) {
                this.error(`Your node must be synced with the Iron Fish network to send a transaction. Please try again later`);
            }
        }
        if (assetId == null) {
            const asset = await (0, asset_1.selectAsset)(client, from, {
                action: 'send',
                showNativeAsset: true,
                showNonCreatorAsset: true,
                showSingleAssetChoice: false,
                confirmations: flags.confirmations,
            });
            assetId = asset?.id;
            if (!assetId) {
                assetId = rust_nodejs_1.Asset.nativeId().toString('hex');
            }
        }
        const assetData = (await client.wallet.getAsset({
            account: from,
            id: assetId,
            confirmations: flags.confirmations,
        })).content;
        let amount;
        if (flags.amount) {
            const [parsedAmount, error] = sdk_1.CurrencyUtils.tryMajorToMinor(flags.amount, assetId, assetData?.verification);
            if (error) {
                this.error(`${error.message}`);
            }
            amount = parsedAmount;
        }
        if (amount == null) {
            amount = await (0, currency_1.promptCurrency)({
                client: client,
                required: true,
                text: 'Enter the amount in the major denomination',
                minimum: 1n,
                logger: this.logger,
                assetId: assetId,
                assetVerification: assetData.verification,
                balance: {
                    account: from,
                    confirmations: flags.confirmations,
                },
            });
        }
        if (!from) {
            const response = await client.wallet.getDefaultAccount();
            if (!response.content.account) {
                this.error(`No account is currently active.
           Use ironfish wallet:create <name> to first create an account`);
            }
            from = response.content.account.name;
        }
        if (!to) {
            to = await core_1.CliUx.ux.prompt('Enter the public address of the recipient', {
                required: true,
            });
        }
        const memo = flags.memo?.trim() ??
            (await core_1.CliUx.ux.prompt('Enter the memo (or leave blank)', { required: false }));
        if (!(0, sdk_1.isValidPublicAddress)(to)) {
            this.log(`A valid public address is required`);
            this.exit(1);
        }
        if (flags.expiration !== undefined && flags.expiration < 0) {
            this.log('Expiration sequence must be non-negative');
            this.exit(1);
        }
        const params = {
            account: from,
            outputs: [
                {
                    publicAddress: to,
                    amount: sdk_1.CurrencyUtils.encode(amount),
                    memo,
                    assetId,
                },
            ],
            fee: flags.fee ? sdk_1.CurrencyUtils.encode(flags.fee) : null,
            feeRate: flags.feeRate ? sdk_1.CurrencyUtils.encode(flags.feeRate) : null,
            expiration: flags.expiration,
            confirmations: flags.confirmations,
            notes: flags.note,
        };
        let raw;
        if (params.fee === null && params.feeRate === null) {
            raw = await (0, fees_1.selectFee)({
                client,
                transaction: params,
                account: from,
                confirmations: flags.confirmations,
                logger: this.logger,
            });
        }
        else {
            const response = await client.wallet.createTransaction(params);
            const bytes = Buffer.from(response.content.transaction, 'hex');
            raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        }
        if (flags.rawTransaction) {
            this.log('Raw Transaction');
            this.log(sdk_1.RawTransactionSerde.serialize(raw).toString('hex'));
            this.log(`Run "ironfish wallet:post" to post the raw transaction. `);
            this.exit(0);
        }
        if (flags.unsignedTransaction) {
            const response = await client.wallet.buildTransaction({
                account: from,
                rawTransaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            });
            this.log('Unsigned Transaction');
            this.log(response.content.unsignedTransaction);
            this.exit(0);
        }
        (0, transaction_1.displayTransactionSummary)(raw, assetData, amount, from, to, memo);
        const spendPostTime = (0, spendPostTime_1.getSpendPostTimeInMs)(this.sdk);
        const transactionTimer = new transaction_1.TransactionTimer(spendPostTime, raw);
        if (spendPostTime > 0) {
            this.log(`Time to send: ${sdk_1.TimeUtils.renderSpan(transactionTimer.getEstimateInMs(), {
                hideMilliseconds: true,
            })}`);
        }
        await (0, utils_1.confirmOperation)({
            confirm: flags.confirm,
            cancelledMessage: 'Transaction aborted.',
        });
        transactionTimer.start();
        const response = await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            account: from,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        transactionTimer.end();
        this.log(`Sending took ${sdk_1.TimeUtils.renderSpan(transactionTimer.getEndTime() - transactionTimer.getStartTime(), {
            hideMilliseconds: true,
        })}`);
        await (0, spendPostTime_1.updateSpendPostTimeInMs)(this.sdk, raw, transactionTimer.getStartTime(), transactionTimer.getEndTime());
        if (response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        const renderedAmount = sdk_1.CurrencyUtils.render(amount, true, assetData.id, assetData.verification);
        const renderedFee = sdk_1.CurrencyUtils.render(transaction.fee(), true);
        this.log(`Sent ${renderedAmount} to ${to} from ${from}`);
        this.log(`Hash: ${transaction.hash().toString('hex')}`);
        this.log(`Fee: ${renderedFee}`);
        this.log(`Memo: ${memo}`);
        const networkId = (await client.chain.getNetworkInfo()).content.networkId;
        const transactionUrl = (0, explorer_1.getExplorer)(networkId)?.getTransactionUrl(transaction.hash().toString('hex'));
        if (transactionUrl) {
            this.log(`\nIf the transaction is mined, it will appear here: ${transactionUrl}`);
        }
        if (flags.watch) {
            this.log('');
            await (0, transaction_1.watchTransaction)({
                client,
                logger: this.logger,
                account: from,
                hash: transaction.hash().toString('hex'),
            });
        }
    }
}
Send.description = `Send coins to another account`;
Send.examples = [
    '$ ironfish wallet:send --amount 2.003 --fee 0.00000001 --to 997c586852d1b12da499bcff53595ba37d04e4909dbdb1a75f3bfd90dd7212217a1c2c0da652d187fc52ed',
    '$ ironfish wallet:send --amount 2.003 --fee 0.00000001 --to 997c586852d1b12da499bcff53595ba37d04e4909dbdb1a75f3bfd90dd7212217a1c2c0da652d187fc52ed --account otheraccount',
    '$ ironfish wallet:send --amount 2.003 --fee 0.00000001 --to 997c586852d1b12da499bcff53595ba37d04e4909dbdb1a75f3bfd90dd7212217a1c2c0da652d187fc52ed --account otheraccount --memo "enjoy!"',
];
Send.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to send money from',
    }),
    amount: (0, flags_1.ValueFlag)({
        char: 'a',
        description: 'The amount to send in the major denomination',
        flagName: 'amount',
    }),
    to: core_1.Flags.string({
        char: 't',
        description: 'The public address of the recipient',
    }),
    fee: (0, flags_1.IronFlag)({
        char: 'o',
        description: 'The fee amount in IRON',
        minimum: 1n,
        flagName: 'fee',
    }),
    feeRate: (0, flags_1.IronFlag)({
        char: 'r',
        description: 'The fee rate amount in IRON/Kilobyte',
        minimum: 1n,
        flagName: 'fee rate',
    }),
    memo: core_1.Flags.string({
        char: 'm',
        description: 'The memo of transaction',
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    watch: core_1.Flags.boolean({
        default: false,
        description: 'Wait for the transaction to be confirmed',
    }),
    expiration: core_1.Flags.integer({
        char: 'e',
        description: 'The block sequence after which the transaction will be removed from the mempool. Set to 0 for no expiration.',
    }),
    confirmations: core_1.Flags.integer({
        char: 'c',
        description: 'Minimum number of block confirmations needed to include a note. Set to 0 to include all blocks.',
        required: false,
    }),
    assetId: (0, flags_1.HexFlag)({
        char: 'i',
        description: 'The identifier for the asset to use when sending',
    }),
    rawTransaction: core_1.Flags.boolean({
        default: false,
        description: 'Return raw transaction. Use it to create a transaction but not post to the network',
    }),
    unsignedTransaction: core_1.Flags.boolean({
        hidden: true,
        default: false,
        description: 'Return a serialized UnsignedTransaction. Use it to create a transaction and build proofs but not post to the network',
        exclusive: ['rawTransaction'],
    }),
    offline: core_1.Flags.boolean({
        default: false,
        description: 'Allow offline transaction creation',
    }),
    note: core_1.Flags.string({
        char: 'n',
        description: 'The note hashes to include in the transaction',
        multiple: true,
    }),
};
exports.Send = Send;
//# sourceMappingURL=send.js.map