"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionViewCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const inquirer_1 = __importDefault(require("inquirer"));
const command_1 = require("../../../command");
const flags_1 = require("../../../flags");
const longPrompt_1 = require("../../../utils/longPrompt");
const transaction_1 = require("../../../utils/transaction");
class TransactionViewCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(TransactionViewCommand);
        const client = await this.sdk.connectRpc();
        const account = flags.account ?? (await this.selectAccount(client));
        let transactionString = flags.transaction;
        if (!transactionString) {
            transactionString = await (0, longPrompt_1.longPrompt)('Enter the hex-encoded raw transaction or unsigned transaction to view', {
                required: true,
            });
        }
        const rawTransaction = this.tryDeserializeRawTransaction(transactionString);
        if (rawTransaction) {
            return await (0, transaction_1.renderRawTransactionDetails)(client, rawTransaction, account, this.logger);
        }
        const unsignedTransaction = this.tryDeserializeUnsignedTransaction(transactionString);
        if (unsignedTransaction) {
            return await (0, transaction_1.renderUnsignedTransactionDetails)(client, unsignedTransaction, account, this.logger);
        }
        this.error('Unable to deserialize transaction input as a raw transacton or an unsigned transaction');
    }
    async selectAccount(client) {
        const accountsResponse = await client.wallet.getAccounts();
        const choices = [];
        for (const account of accountsResponse.content.accounts) {
            choices.push({
                account,
                value: account,
            });
        }
        choices.sort();
        const selection = await inquirer_1.default.prompt([
            {
                name: 'account',
                message: 'Select account',
                type: 'list',
                choices,
            },
        ]);
        return selection.account;
    }
    tryDeserializeRawTransaction(transaction) {
        try {
            return sdk_1.RawTransactionSerde.deserialize(Buffer.from(transaction, 'hex'));
        }
        catch (e) {
            this.logger.debug(`Failed to deserialize transaction as RawTransaction: ${sdk_1.ErrorUtils.renderError(e)}`);
            return undefined;
        }
    }
    tryDeserializeUnsignedTransaction(transaction) {
        try {
            return new sdk_1.UnsignedTransaction(Buffer.from(transaction, 'hex'));
        }
        catch (e) {
            this.logger.debug(`Failed to deserialize transaction as UnsignedTransaction: ${sdk_1.ErrorUtils.renderError(e)}`);
            return undefined;
        }
    }
}
TransactionViewCommand.description = `View transaction details`;
TransactionViewCommand.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The name of the account to use to for viewing transaction details',
    }),
    transaction: core_1.Flags.string({
        char: 't',
        description: 'The hex-encoded raw transaction or unsigned transaction to view',
    }),
};
exports.TransactionViewCommand = TransactionViewCommand;
//# sourceMappingURL=view.js.map