"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.benchmarkSpendPostTime = exports.getSpendPostTimeInMs = exports.updateSpendPostTimeInMs = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const note_1 = require("./note");
/**
 * Recalculates the average spendPostTime based on the new measurement.
 */
async function updateSpendPostTimeInMs(sdk, raw, startTime, endTime) {
    if (raw.spends.length === 0) {
        return;
    }
    const transactionDuration = endTime - startTime;
    const averageSpendTime = Math.ceil(transactionDuration / raw.spends.length);
    const oldAverage = sdk.internal.get('spendPostTime');
    const oldMeasurementCount = sdk.internal.get('spendPostTimeMeasurements');
    // Calculate the new average using the formula: ((oldAverage * oldCount) + newValue) / newCount
    const newMeasurementCount = oldMeasurementCount + 1;
    const newAverageSpendPostTime = (oldAverage * oldMeasurementCount + averageSpendTime) / newMeasurementCount;
    sdk.internal.set('spendPostTime', newAverageSpendPostTime);
    sdk.internal.set('spendPostTimeMeasurements', newMeasurementCount);
    await sdk.internal.save();
}
exports.updateSpendPostTimeInMs = updateSpendPostTimeInMs;
function getSpendPostTimeInMs(sdk) {
    return sdk.internal.get('spendPostTime');
}
exports.getSpendPostTimeInMs = getSpendPostTimeInMs;
async function benchmarkSpendPostTime(sdk, client, account) {
    core_1.CliUx.ux.action.start('Measuring time to combine 1 note');
    const publicKey = (await client.wallet.getAccountPublicKey({
        account: account,
    })).content.publicKey;
    const notes = await (0, note_1.fetchNotes)(client, account, 10);
    // Not enough notes in the account to measure the time to combine a note
    if (notes.length < 3) {
        core_1.CliUx.ux.error('Not enough notes.');
    }
    const feeRates = await client.wallet.estimateFeeRates();
    /** Transaction 1: selects 1 note */
    const txn1Params = {
        account: account,
        outputs: [
            {
                publicAddress: publicKey,
                amount: sdk_1.CurrencyUtils.encode(BigInt(notes[0].value)),
                memo: '',
            },
        ],
        fee: null,
        feeRate: null,
        notes: [notes[0].noteHash],
    };
    /** Transaction 2: selects two notes */
    const txn2Params = {
        account: account,
        outputs: [
            {
                publicAddress: publicKey,
                amount: sdk_1.CurrencyUtils.encode(BigInt(notes[0].value) + BigInt(notes[1].value)),
                memo: '',
            },
        ],
        fee: null,
        feeRate: null,
        notes: [notes[0].noteHash, notes[1].noteHash],
    };
    const promisesTxn1 = [];
    const promisesTxn2 = [];
    for (let i = 0; i < 3; i++) {
        promisesTxn1.push(measureTransactionPostTime(client, txn1Params, feeRates));
        promisesTxn2.push(measureTransactionPostTime(client, txn2Params, feeRates));
    }
    const resultTxn1 = await Promise.all(promisesTxn1);
    const resultTxn2 = await Promise.all(promisesTxn2);
    const spendPostTime = Math.ceil((resultTxn2.reduce((acc, curr) => acc + curr, 0) -
        resultTxn1.reduce((acc, curr) => acc + curr, 0)) /
        3);
    if (spendPostTime <= 0) {
        core_1.CliUx.ux.error('Error calculating spendPostTime. Please try again.');
    }
    core_1.CliUx.ux.action.stop(sdk_1.TimeUtils.renderSpan(spendPostTime));
    sdk.internal.set('spendPostTime', spendPostTime);
    sdk.internal.set('spendPostTimeMeasurements', 1);
    await sdk.internal.save();
    return spendPostTime;
}
exports.benchmarkSpendPostTime = benchmarkSpendPostTime;
async function measureTransactionPostTime(client, params, feeRates) {
    const response = await client.wallet.createTransaction({
        ...params,
        feeRate: feeRates.content.fast,
    });
    const bytes = Buffer.from(response.content.transaction, 'hex');
    const raw = sdk_1.RawTransactionSerde.deserialize(bytes);
    const start = sdk_1.BenchUtils.start();
    await client.wallet.postTransaction({
        transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
        broadcast: false,
    });
    return sdk_1.BenchUtils.end(start);
}
//# sourceMappingURL=spendPostTime.js.map