"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchTransaction = exports.displayTransactionSummary = exports.renderRawTransactionDetails = exports.renderUnsignedTransactionDetails = exports.TransactionTimer = void 0;
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const asset_1 = require("./asset");
class TransactionTimer {
    constructor(spendPostTime, raw) {
        // if spendPostTime is 0, we don't have enough data to estimate the time to send a transaction
        this.estimateInMs =
            spendPostTime > 0 ? Math.max(Math.ceil(spendPostTime * raw.spends.length), 1000) : -1;
    }
    getEstimateInMs() {
        return this.estimateInMs;
    }
    getStartTime() {
        if (!this.startTime) {
            throw new Error('TransactionTimer not started');
        }
        return this.startTime;
    }
    getEndTime() {
        if (!this.endTime) {
            throw new Error('TransactionTimer not ended');
        }
        return this.endTime;
    }
    start() {
        this.startTime = performance.now();
        if (this.estimateInMs <= 0) {
            core_1.CliUx.ux.action.start('Sending the transaction');
            return;
        }
        this.progressBar = core_1.CliUx.ux.progress({
            format: '{title}: [{bar}] {percentage}% | {estimate}',
        });
        this.progressBar.start(100, 0, {
            title: 'Sending the transaction',
            estimate: sdk_1.TimeUtils.renderSpan(this.estimateInMs, { hideMilliseconds: true }),
        });
        this.timer = setInterval(() => {
            if (!this.progressBar || !this.startTime) {
                return;
            }
            const durationInMs = performance.now() - this.startTime;
            const timeRemaining = this.estimateInMs - durationInMs;
            const progress = Math.round((durationInMs / this.estimateInMs) * 100);
            this.progressBar.update(progress, {
                estimate: sdk_1.TimeUtils.renderSpan(timeRemaining, { hideMilliseconds: true }),
            });
        }, 1000);
    }
    end() {
        if (!this.startTime) {
            // transaction timer has not been started
            return;
        }
        this.endTime = performance.now();
        if (!this.progressBar || !this.timer || this.estimateInMs <= 0) {
            core_1.CliUx.ux.action.stop();
            return;
        }
        clearInterval(this.timer);
        this.progressBar.update(100, {
            estimate: 'done',
        });
        this.progressBar.stop();
    }
}
exports.TransactionTimer = TransactionTimer;
async function renderUnsignedTransactionDetails(client, unsignedTransaction, account, logger) {
    logger = logger ?? (0, sdk_1.createRootLogger)();
    let response;
    if (unsignedTransaction.notes.length > 0) {
        response = await client.wallet.getUnsignedTransactionNotes({
            account,
            unsignedTransaction: unsignedTransaction.serialize().toString('hex'),
        });
    }
    const assetIds = collectAssetIds(unsignedTransaction, response?.content);
    const assetLookup = await (0, asset_1.getAssetsByIDs)(client, assetIds, account, undefined);
    if (unsignedTransaction.mints.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log('Transaction Mints:');
        logger.log('==================');
        for (const [i, mint] of unsignedTransaction.mints.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const renderedAmount = sdk_1.CurrencyUtils.render(mint.value, false, mint.asset.id().toString('hex'), assetLookup[mint.asset.id().toString('hex')].verification);
            logger.log(`Asset ID:      ${mint.asset.id().toString('hex')}`);
            logger.log(`Name:          ${mint.asset.name().toString('utf8')}`);
            logger.log(`Amount:        ${renderedAmount}`);
            if (mint.transferOwnershipTo) {
                logger.log(`Ownership of logger asset will be transferred to ${mint.transferOwnershipTo.toString('hex')}. The current account will no longer have any permission to mint or modify logger asset. logger cannot be undone.`);
            }
            logger.log('');
        }
    }
    if (unsignedTransaction.burns.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log('Transaction Burns:');
        logger.log('==================');
        for (const [i, burn] of unsignedTransaction.burns.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const renderedAmount = sdk_1.CurrencyUtils.render(burn.value, false, burn.assetId.toString('hex'), assetLookup[burn.assetId.toString('hex')].verification);
            logger.log(`Asset ID:      ${burn.assetId.toString('hex')}`);
            logger.log(`Amount:        ${renderedAmount}`);
            logger.log('');
        }
    }
    if (unsignedTransaction.notes.length > 0) {
        const response = await client.wallet.getUnsignedTransactionNotes({
            account,
            unsignedTransaction: unsignedTransaction.serialize().toString('hex'),
        });
        logger.log('');
        logger.log('==================');
        logger.log('Notes sent:');
        logger.log('==================');
        for (const [i, note] of response.content.sentNotes.entries()) {
            // Skip logger since we'll re-render for received notes
            if (note.owner === note.sender) {
                continue;
            }
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const renderedAmount = sdk_1.CurrencyUtils.render(note.value, true, note.assetId, assetLookup[note.assetId].verification);
            logger.log(`Amount:        ${renderedAmount}`);
            logger.log(`Memo:          ${note.memo}`);
            logger.log(`Recipient:     ${note.owner}`);
            logger.log(`Sender:        ${note.sender}`);
            logger.log('');
        }
        logger.log('');
        logger.log('==================');
        logger.log('Notes received:');
        logger.log('==================');
        for (const [i, note] of response.content.receivedNotes.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const renderedAmount = sdk_1.CurrencyUtils.render(note.value, true, note.assetId, assetLookup[note.assetId].verification);
            logger.log(`Amount:        ${renderedAmount}`);
            logger.log(`Memo:          ${note.memo}`);
            logger.log(`Recipient:     ${note.owner}`);
            logger.log(`Sender:        ${note.sender}`);
            logger.log('');
        }
        if (!response.content.sentNotes.length && !response.content.receivedNotes.length) {
            logger.log('');
            logger.log('------------------');
            logger.log('Account unable to decrypt any notes in this transaction');
            logger.log('------------------');
        }
    }
    logger.log('');
}
exports.renderUnsignedTransactionDetails = renderUnsignedTransactionDetails;
async function renderRawTransactionDetails(client, rawTransaction, account, logger) {
    const assetIds = collectRawTransactionAssetIds(rawTransaction);
    const assetLookup = await (0, asset_1.getAssetVerificationByIds)(client, assetIds, account, undefined);
    const feeString = sdk_1.CurrencyUtils.render(rawTransaction.fee, true);
    // Every transaction except for a miners transaction should have at least 1 spend for the transaction fee
    const from = rawTransaction.spends.length ? rawTransaction.spends[0].note.owner() : null;
    const summary = `\
\n===================
Transaction Summary
===================

From            ${from}
Fee             ${feeString}
Expiration      ${rawTransaction.expiration !== null ? rawTransaction.expiration.toString() : ''}`;
    logger.log(summary);
    if (rawTransaction.mints.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log(`Mints (${rawTransaction.mints.length})`);
        logger.log('==================');
        for (const [i, mint] of rawTransaction.mints.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const asset = new rust_nodejs_1.Asset(mint.creator, mint.name, mint.metadata);
            const renderedAmount = sdk_1.CurrencyUtils.render(mint.value, false, asset.id().toString('hex'), assetLookup[asset.id().toString('hex')]);
            logger.log(`Amount         ${renderedAmount}`);
            logger.log(`Asset ID       ${asset.id().toString('hex')}`);
            logger.log(`Name           ${mint.name}`);
            logger.log(`Metadata       ${mint.metadata}`);
            if (mint.transferOwnershipTo) {
                logger.log(`Ownership of asset will be transferred to ${mint.transferOwnershipTo}. The current account will no longer have any permission to mint or modify asset. This action cannot be undone.`);
            }
            logger.log('');
        }
    }
    if (rawTransaction.burns.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log(`Burns (${rawTransaction.burns.length})`);
        logger.log('==================');
        for (const [i, burn] of rawTransaction.burns.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const renderedAmount = sdk_1.CurrencyUtils.render(burn.value, false, burn.assetId.toString('hex'), assetLookup[burn.assetId.toString('hex')]);
            logger.log(`Amount         ${renderedAmount}`);
            logger.log(`Asset ID       ${burn.assetId.toString('hex')}`);
            logger.log('');
        }
    }
    if (rawTransaction.spends.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log(`Spends (${rawTransaction.spends.length})`);
        logger.log('==================');
        for (const [i, { note }] of rawTransaction.spends.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const { symbol } = (0, sdk_1.assetMetadataWithDefaults)(note.assetId().toString('hex'), assetLookup[note.assetId().toString('hex')]);
            logger.log(`Asset           ${symbol}`);
            logger.log(`Note Hash       ${note.hash().toString('hex')}`);
            logger.log('');
        }
    }
    if (rawTransaction.outputs.length > 0) {
        logger.log('');
        logger.log('==================');
        logger.log(`Notes (${rawTransaction.outputs.length}) (Additional notes will be added to return unspent assets to the sender)`);
        logger.log('==================');
        for (const [i, { note }] of rawTransaction.outputs.entries()) {
            if (i !== 0) {
                logger.log('------------------');
            }
            logger.log('');
            const { symbol } = (0, sdk_1.assetMetadataWithDefaults)(note.assetId().toString('hex'), assetLookup[note.assetId().toString('hex')]);
            const renderedAmount = sdk_1.CurrencyUtils.render(note.value(), false, note.assetId().toString('hex'), assetLookup[note.assetId().toString('hex')]);
            logger.log(`Amount         ${renderedAmount}`);
            logger.log(`Asset          ${symbol}`);
            logger.log(`Memo           ${note.memo().toString('utf-8')}`);
            logger.log(`Recipient      ${note.owner()}`);
            logger.log(`Sender         ${note.sender()}`);
            logger.log('');
        }
    }
    logger.log('');
}
exports.renderRawTransactionDetails = renderRawTransactionDetails;
function displayTransactionSummary(transaction, asset, amount, from, to, memo, logger) {
    logger = logger ?? (0, sdk_1.createRootLogger)();
    const amountString = sdk_1.CurrencyUtils.render(amount, true, asset.id, asset.verification);
    const feeString = sdk_1.CurrencyUtils.render(transaction.fee, true);
    const summary = `\
\nTRANSACTION SUMMARY:
From                 ${from}
To                   ${to}
Amount               ${amountString}
Fee                  ${feeString}
Memo                 ${memo}
Outputs              ${transaction.outputs.length}
Spends               ${transaction.spends.length}
Expiration           ${transaction.expiration ? transaction.expiration.toString() : ''}
`;
    logger.log(summary);
}
exports.displayTransactionSummary = displayTransactionSummary;
async function watchTransaction(options) {
    const logger = options.logger ?? (0, sdk_1.createRootLogger)();
    const waitUntil = options.waitUntil ?? sdk_1.TransactionStatus.CONFIRMED;
    const pollFrequencyMs = options.pollFrequencyMs ?? 10000;
    let lastTime = Date.now();
    let last = await options.client.wallet.getAccountTransaction({
        account: options.account,
        hash: options.hash,
        confirmations: options.confirmations,
    });
    const startTime = lastTime;
    let prevStatus = last?.content.transaction?.status ?? 'not found';
    let currentStatus = prevStatus;
    // If the transaction is already in the desired state, return
    if (currentStatus === waitUntil) {
        logger.log(`Transaction ${options.hash} is ${waitUntil}`);
        return;
    }
    logger.log(`Watching transaction ${options.hash}`);
    core_1.CliUx.ux.action.start(`Current Status`);
    const span = sdk_1.TimeUtils.renderSpan(0, { hideMilliseconds: true });
    core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const response = await options.client.wallet.getAccountTransaction({
            account: options.account,
            hash: options.hash,
            confirmations: options.confirmations,
        });
        currentStatus = response?.content.transaction?.status ?? 'not found';
        if (prevStatus !== 'not found' && currentStatus === 'not found') {
            core_1.CliUx.ux.action.stop(`Transaction ${options.hash} deleted while watching it.`);
            break;
        }
        if (currentStatus === prevStatus) {
            const duration = Date.now() - lastTime;
            const span = sdk_1.TimeUtils.renderSpan(duration, { hideMilliseconds: true });
            core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
            await sdk_1.PromiseUtils.sleep(pollFrequencyMs);
            continue;
        }
        // State has changed
        const now = Date.now();
        const duration = now - lastTime;
        lastTime = now;
        core_1.CliUx.ux.action.stop(`${prevStatus} -> ${currentStatus}: ${sdk_1.TimeUtils.renderSpan(duration, {
            hideMilliseconds: true,
        })}`);
        last = response;
        prevStatus = currentStatus;
        core_1.CliUx.ux.action.start(`Current Status`);
        const span = sdk_1.TimeUtils.renderSpan(0, { hideMilliseconds: true });
        core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
        if (currentStatus === waitUntil) {
            const duration = now - startTime;
            const span = sdk_1.TimeUtils.renderSpan(duration, { hideMilliseconds: true });
            core_1.CliUx.ux.action.stop(`done after ${span}`);
            break;
        }
    }
}
exports.watchTransaction = watchTransaction;
function collectRawTransactionAssetIds(rawTransaction) {
    const assetIds = new Set();
    for (const mint of rawTransaction.mints) {
        const newAsset = new rust_nodejs_1.Asset(mint.creator, mint.name, mint.metadata);
        assetIds.add(newAsset.id().toString('hex'));
    }
    for (const burn of rawTransaction.burns) {
        assetIds.add(burn.assetId.toString('hex'));
    }
    for (const spend of rawTransaction.spends) {
        assetIds.add(spend.note.assetId().toString('hex'));
    }
    for (const output of rawTransaction.outputs) {
        assetIds.add(output.note.assetId().toString('hex'));
    }
    return Array.from(assetIds);
}
function collectAssetIds(unsignedTransaction, notes) {
    const assetIds = new Set();
    for (const mint of unsignedTransaction.mints) {
        assetIds.add(mint.asset.id().toString('hex'));
    }
    for (const burn of unsignedTransaction.burns) {
        assetIds.add(burn.assetId.toString('hex'));
    }
    if (notes) {
        for (const receivedNote of notes.receivedNotes) {
            assetIds.add(receivedNote.assetId);
        }
        for (const sentNotes of notes.sentNotes) {
            assetIds.add(sentNotes.assetId);
        }
    }
    return Array.from(assetIds);
}
//# sourceMappingURL=transaction.js.map